package Grid::GPM::ExpandSource; 
use strict; 
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;
require Grid::GPM::Archive;
require Archive::Tar;
require AutoLoader;
use Data::Dumper;
use Grid::GPT::Package;
use Grid::GPM::FilelistFunctions;
use Grid::GPM::BuildEnv;
use Grid::GPM::Inform;
use Cwd;
use File::Find;
use Carp;

@ISA = qw(Exporter AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw();
$VERSION = '0.01';

# Preloaded methods go here.
sub open_bundle {
  my (%arg) = @_;
  my $startdir = cwd();

  my $builddir = defined $arg{'builddir'} ? $arg{'builddir'} : $startdir;
  if ($arg{'file'} =~ m!_bundle!) {
    my $file = Grid::GPM::FilelistFunctions::abspath($arg{'file'});

    chdir $builddir;
    my @list = untar($file);

    my @pl = grep {m!packaging_list!} @list;

    die "ERROR: packaging list not found for bundle $arg{'file'}\n" if ! @pl;

    open(PL, "$pl[0]") ||
      die "ERROR: $pl[0] could not be accessed for bundle $arg{'file'}\n";

    chdir $startdir;

    my @sources;

    for my $l (<PL>) {
      next if $l !~ m!\w!;
      chomp $l;
      my @tarfiles = grep { m!$l\-\d! } @list;

      if (! @tarfiles) {
        print "ERROR: Archive not found for package $l\nArchive:\n";
        for (@list) {
          print "\t$_\n";
        }
        exit 1;
      }
      my $src = new Grid::GPM::ExpandSource(
                                            tarfile => "$builddir/$tarfiles[0]",
                                            builddir => $arg{'builddir'});
      push @sources, $src;
    }

    close PL;

    return \@sources;

  } else {
    my $src = new Grid::GPM::ExpandSource(tarfile => $arg{'file'}, 
                                          builddir => $arg{'builddir'});
    return [ $src ];
  }
}

sub new {
  my ($class, %arg) = @_;
  # set the following directories
  # patch directory.
  # source directory.
  
  my $me = {
            srcdir => $arg{'srcdir'},
            srcfile => $arg{'srcfile'},
            patchdir => $arg{'patchdir'},
            builddir => $arg{'builddir'},
            tarfile => $arg{'tarfile'}
	   };
  
  $me->{'globusdir'} = $ENV{GLOBUS_LOCATION};
  
  Grid::GPM::FilelistFunctions::mkinstalldir($me->{'builddir'}) 
      if defined $me->{'builddir'};
  
  $|++; # need for timely print outputs
  
  for my $f ('srcdir', 'patchdir', 'builddir', 'tarfile', 'srcfile') {
    next if ! defined $me->{$f};
    my ($path, $last) = ($me->{$f}, "");
    $path =~ s!([^/]+)$!!;
    $last = $1;
    $path = "." if $path !~ m!\S!;
    $path = Grid::GPM::FilelistFunctions::abspath($path);
    $path = Cwd::abs_path($path) || die "$path could not be accessed\n"; 
    
    $me->{$f} = "$path/$last";
    die "ERROR: $me->{$f} is not a valid path\n" 
      if ! defined -f $me->{$f};
  }
  
  bless $me, $class;
  
  return $me;
}

sub setup_source {
  my ($me) = @_;
   my $startdir = cwd();
  $me->{'srcdir'} = $startdir if ! defined $me->{'srcdir'};

  $me->expand() if defined $me->{'tarfile'};

  opendir(DIR, $me->{'srcdir'}) 
    or die "ERROR can't access $me->{'srcdir'}\n";
  my @contents = readdir(DIR);

  if (-f "$me->{'srcdir'}/globus_build_instructions") {
    open (INSTR, "$me->{'srcdir'}/globus_build_instructions") 
      or die "could not open globus_build_instructions\n";
    
    $me->{'build_instructions'} = [ <INSTR> ];
    close INSTR;
  }

  if (! defined ($me->{'patchdir'})) {
    my $patchdir = findindir(qr/patches/, "patch directories",\@contents);
    $me->{'patchdir'} = Cwd::abs_path("$me->{'srcdir'}/$patchdir") 
      if defined $patchdir;
  }
  if (! defined $me->{'srcfile'}) {

    my $pkgdata = findindir(qr/pkgdata/, "pkgdata directory",\@contents);

    if (defined ($pkgdata)) {

      $me->{'srcfile'} = "$me->{'srcdir'}/pkgdata/pkg_data_src.gpt.in";
      die "ERROR: Source metadata file not found\n" if ! -f $me->{'srcfile'};

    } else {

      $me->{'srcfile'} = "$me->{'srcdir'}/pkg_data_src.gpt";
      die "ERROR: Source metadata file not found\n" 
        if ! defined $me->{'srcfile'};

    }

  }

  if (! defined ($me->{'filelist'})) {
    my $filelist = findindir(qr/filelist/, "filelists",\@contents);
    if (defined ($filelist)) {
      open(FILE, "$me->{'srcdir'}/$filelist") || 
        die "ERROR: cannot open $me->{'srcdir'}/filelist\n";
      $me->{'filelist'} = [ <FILE> ];
      close FILE;
    }
  }

  if (! -f "$me->{'srcdir'}/Makefile.in" and ! -f "$me->{'srcdir'}/INSTALL") {
    my @subdirs = 
      grep { -f "$me->{'srcdir'}/$_/Makefile.in" or -f "$me->{'srcdir'}/$_/INSTALL"}
        grep { -d "$me->{'srcdir'}/$_" } 
          @contents;
    die "ERROR: Can't find sources in $me->{'srcdir'}\n" if ! @subdirs;
    die "ERROR: Found multiple sources in $me->{'srcdir'}/[ @subdirs ]\n" if @subdirs > 1;

    $me->{'srcdir'} .= "/$subdirs[0]";
  }


}

sub findindir{
  my ($expression, $lookfor, $contents) = @_;
  my @hits = grep {m!$expression!} @$contents;
  die "ERROR: Too many $lookfor found\n" if @hits > 1;
  return $hits[0] if @hits;
  return undef;
};


sub expand {
  my ($me) = @_;
  my $startdir = cwd();
  my $builddir = $startdir;
  $builddir = $me->{'builddir'} if defined $me->{'builddir'};
  
  return if !defined $me->{'tarfile'};
  
  # need to untar a source tarball
  
  chdir $builddir;
  
  print "UNPACKING $me->{'tarfile'}\n";
  
  my @list = untar($me->{'tarfile'});
  my $srcdir = $list[0];
  $srcdir =~ s!(^[^/]+)/.+!$1!;
  
  $me->{'srcdir'} = "$builddir/$srcdir";
  chdir $startdir; 
}

sub expand_unpatched_source {
  my ($me, $tarfile) = @_;
 
  my $startdir = cwd();
  croak "I'm untarring source\n"; 
  chdir $me->{'srcdir'};

  my $unpatched_srcdir;
  
  print "UNPACKING $tarfile\n";
  my @list = untar($tarfile);
  chdir $startdir; 
}

sub untar {
  my($file) = @_;
  my $cmd;
  if ($file =~ m!\.gz$!) {
    $cmd = Grid::GPM::BuildEnv::gunzip() . " < $file | " .
        Grid::GPM::BuildEnv::tar() . " xvf -";
  }
  else
  {
    $cmd = Grid::GPM::BuildEnv::tar() . " xvf $file";
  }
  my @list = `$cmd 2>&1`;
  die "$file could not be untarred:$?\n" if ($?);
  for (@list) {
    s!^(?:x\s+)?(?:\./)?!!;
    s!,.+$!!;
    chomp;
  }
  return @list;
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

Grid::GPM:ExpandSource - Perl extension for expanding source packages
and bundles.

=head1 SYNOPSIS

  use ExpandSource;
  my $src = new Grid::GPM::ExpandSource(srcdir => $s,
                                        srcfile => $srcfile,
                                       );
  my @sources = Grid::GPM::ExpandSource::open_bundle(file => "$startdir/$t", 
                                                     builddir => $tmpdir);

 for my $o (@sources) {
   $o->setup();
 }

=head1 DESCRIPTION

B<Grid::GPM::ExpandSource> is used to unarchive a source package or bundle and prepare the contents for building.


=head1 METHODS

=over 4

=item open_bundle(file => "$startdir/$t", builddir => $tmpdir)

This is a class function that untars a source bundle and returns a
list of ExpandSource objects.  Each object represents a source
package.  A bundle is an archive of source packages with a file called
I<packaging_list> which lists the packages in build dependent order.
For example here is the contents of the globus_gsi_bundle.tar.gz:

   -rw-r--r-- mbletzin/mbletzin 152868 2001-09-25 10:59:03 ./gsi/globus_core-2.0.tar.gz
   -rw-r--r-- mbletzin/mbletzin 8557949 2001-09-25 10:50:02 ./gsi/globus_openssl-0.9.6b.tar.gz
   -rw-r--r-- mbletzin/mbletzin  120013 2001-09-25 10:59:04 ./gsi/globus_core_setup-2.0.tar.gz
   -rw-r--r-- mbletzin/mbletzin  248338 2001-09-25 10:59:06 ./gsi/globus_ssl_utils-2.0.tar.gz
   -rw-r--r-- mbletzin/mbletzin  161845 2001-09-25 10:59:09 ./gsi/globus_gssapi_gsi-2.0.tar.gz
   -rw-r--r-- mbletzin/mbletzin  120526 2001-09-25 10:59:10 ./gsi/globus_gss_assist-2.0.tar.gz
   -rw-r--r-- mbletzin/mbletzin      52 2001-09-25 10:59:10 ./gsi/packaging_list

The contents of the packaging_list file is:

   core
   core_setup
   openssl
   ssl_utils
   gssapi_gsi
   gss_assist

=item setup_source

This object function expands the source tarball if necessary and sets
the attributes that describe the build directory.


=back

=head1 ATTRIBUTES

=over 4

=item srcdir

Points to the top of the source directory

=item srcfile

Points to the location of the source package metadata.

=item patchdir

Points to the directory containing patches for the source packages.
This should not be used for now because of portability problems with
the patch command.

=item builddir

Directory in which the sources are expanded.

=item tarfile

Name of the source package to be expanded.

=item globusdir

Location where the globus packaging tools are installed.

=item build_instructions

Contents of the build_instructions file.  Currently this is not being
used.

=item filelist

Contents of the filelist file.  The file is needed for those packages
which don't generate filelists as part of their build process.

=back

=head1 AUTHOR

Michael Bletzinger <mbletzin@ncsa.uiuc,edu>

=head1 SEE ALSO

perl(1) globus-build(1).

=cut
