package Grid::GPT::BaseDependency;

use strict;
use Carp;

require Exporter;
use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS %binary_dependencies);
use Data::Dumper;
use Grid::GPT::XML;
use Grid::GPT::Definitions;
use Grid::GPT::Version;
require Grid::GPT::BinaryDependency;
require Grid::GPT::SourceDependency;

# set the version for version checking
$VERSION     = 0.01;

@ISA         = qw(Exporter);

sub add_xml_to {
  my (%args) = @_;
  my ($obj, $my_pkg_type, $depshash) = ($args{'xml'}, 
                                        $args{'my_pkg_type'}, 
                                        $args{'depshash'});
  die "ERROR: Grid::GPT::XML object does not contain Dependencies\n"
    if $obj->{'name'} !~ m!_Dependencies!;


  my $depname = "Grid::GPT::BinaryDependency";
  $depname = "Grid::GPT::SourceDependency" 
    if $obj->{'name'} =~ m!Source!;
  
  my $type = $obj->{'attributes'}->{'Type'};
  my $deps = $obj->{'contents'};
  for my $d (@$deps) {
    next if ref($d) ne 'HASH';
    my $name = $d->{'attributes'}->{'Name'};
    my $pkg_type = $d->{'attributes'}->{'Package'};
    my $xml_vers = $d->{'contents'};
    my $versions;
    for my $v (@$xml_vers) {
      next if ref($v) ne 'HASH';
      $versions = Grid::GPT::Version::create_version_list($v);
      last;
    }
    die "ERROR: $type dependency $name is missing a version requirement\n" 
      if ! defined $versions;
    my $dep = new $depname(versions => $versions,
                           name => $name,
                           type => $type,
                           pkg_type => $pkg_type,
                           my_pkg_type => $my_pkg_type);
    $dep->add_to($depshash);
  }
}

sub get_xml_from {
  my ($depshash, $xml, $depname) = @_;
  for my $h (sort keys %$depshash) {

    $xml->startTag($depname, Type => $h);
    $xml->characters("\n");
    for my $d (sort keys %{$depshash->{$h}}) {
      $depshash->{$h}->{$d}->write_tag($xml);
    }
    $xml->endTag($depname);
    $xml->characters("\n");
  }
}


sub new {
  my ($that, %arg)  = @_;
  my $class = ref($that) || $that;
  my $self = {};
  bless $self, $class;
  return $self;
}

sub add_to {
  my ($self, $depshash) = @_;
  my ($name, $pkgtype, $type) = ($self->{'name'}, 
                                 $self->{'pkg_type'}, 
                                 $self->{'type'});
  $pkgtype = "" if ! defined $pkgtype;
  $depshash->{$type} = {} if ! defined $depshash->{$type};

  die "ERROR: $name is already defined as a dependency\n"
    if defined $depshash->{$type}->{"$ {name}_$pkgtype"};

  $depshash->{$type}->{"$ {name}_$pkgtype"} = $self;
}

sub clone {
    my $self = shift;
    my $clone = new();
    replicate($self, $clone);
    return $clone;
}

sub replicate {
  my ($rold) = @_;
  if (ref(\$rold) eq 'SCALAR') {
    return $rold;
  } elsif (ref($rold) eq 'ARRAY') {
    my @list = @$rold;
    return \@list;
  } elsif (ref($rold) eq 'HASH') {
    my $rnew = {};
    for my $e (sort keys %$rold) {
      $rnew->{$e} = replicate($rold->{$e});
    }
    return $rnew
  }
}

sub AUTOLOAD {
  use vars qw($AUTOLOAD);
  my $self = shift;
  my $type = ref($self) || croak "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;   # strip fully-qualified portion
  unless (exists $self->{$name} ) {
    croak "Can't access `$name' field in object of class $type";
  } 
  if (@_) {
    return $self->{$name} = shift;
  } else {
    return $self->{$name};
  } 
}

sub write_tag {
  my ($self, $xml) = @_;

  if (defined $self->{'pkg_type'}) {
    $xml->startTag('Dependency',Name => $self->{'name'}, 
		   Package => $self->{'pkg_type'});
  } else {
    $xml->startTag('Dependency',Name => $self->{'name'});    
  }
  $xml->characters("\n");
  Grid::GPT::Version::convert_version_list2xml($self->{'versions'}, $xml);
  $xml->endTag('Dependency');
  $xml->characters("\n");
}

sub DESTROY {}
END { }       # module clean-up code here (global destructor)

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

Grid::GPT::BaseDependency - Perl extension for managing the dependencies in binary packages

=head1 SYNOPSIS

  use Grid::GPT::BaseDependency;
  my $dep = new Grid::GPT::BaseDependency(versions => \@versions, 
						       name => $name,
						       type => $type,
						       pkg_type => $pkg_type,
						       my_pkg_type => $my_pkg_type);
  my $result = $dep->fulfills_dependency($name, $version, $pkg_type);

=head1 DESCRIPTION

I<Grid::GPT::BaseDependency> is used to encapsulate a dependency
that one binary package has to another dependency.  These dependencies
are seperated into the following types:

=over 4

=item Compile

Dependency occurs when the package is used for compiling.  Usually
caused by header files including headers from other packages.

=item Build_Link

Dependency occurs when the package is linked to other applications.
This commonly known as dependent libraries.  

=item Regeneration

Dependency occurs when a statically built package needs to be rebuilt
because of updates to dependent packages.  This results in a new
binary package even though nothing inside the package has changed and
the version number has not been updated.

=item Runtime_Link

Dependency occurs when a package needs to load another package's binary at run-time.

=item Runtime

Dependency occurs when a package needs to read a file or execute a
program from another package.

=back

=head1 Methods

=over 4

=item new

Create a new I<Grid::GPT::BaseDependency> object.  The function has the following named objects:

=over 4

=item versions

Reference to an array of L<Grid::GPT::Version|Grid::GPT::Version> objects.

=item name

Name of the dependent package.

=item type

The type of dependency.

=item pkg_type

The binary package type of the dependent package.

=item my_pkg_type

The binary package type of the package owning this dependency.

=back

=item fulfills_dependency(name, version, pkg_type)

Returns a 1 if the arguments met the requirements of the
dependency. Returns a 0 if not.  Note that package types pgm and
pgm_static are considered equivalent.

=item write_tag(xml_obj)

Adds dependency contents into an L<Grid::GPT::XML|Grid::GPT::XML> object. 


=item convert_dependency_hash2xml(dependency_hash_reference, xml_obj)

Class function which adds the contents of all dependency objects in a
hash reference to an L<Grid::GPT::XML|Grid::GPT::XML> object.

=item create_dependency_hash(xml_obj, package_type_of_dependency_owner)

This is a class function which creates a hash of
I<Grid::GPT::BaseDependency> objects out of an
L<Grid::GPT::XML|Grid::GPT::XML> object.  The key to each hash entry
is of the form <name>_<pkg_type>.

=back

=head1 ToDo

=over 4

=item The internal validate function has not been tested. 

=back

=head1 AUTHOR

Eric Blau <eblau@ncsa.uiuc.edu> Michael Bletzinger <mbletzin@ncsa.uiuc,edu>

=head1 SEE ALSO

perl(1) Grid::GPT::XML(1) Grid::GPT::Version(1).

=cut
